@@ -1,17 +1,24 @@
 package Complete::Riap;
 
-our $DATE = '2014-12-02'; # DATE
-our $VERSION = '0.01'; # VERSION
+our $DATE = '2014-12-25'; # DATE
+our $VERSION = '0.02'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 
+use Complete;
+
 our %SPEC;
 require Exporter;
 our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(complete_riap_url);
 
+$SPEC{':package'} = {
+    v => 1.1,
+    summary => 'Riap-related completion routines',
+};
+
 $SPEC{complete_riap_url} = {
     v => 1.1,
     summary => 'Complete Riap URL',
@@ -27,6 +34,16 @@ _
             req => 1,
             pos => 0,
         },
+        ci => {
+            summary => 'Whether to do case-insensitive search',
+            schema  => 'bool*',
+        },
+        map_case => {
+            schema => 'bool',
+        },
+        exp_im_path => {
+            schema => 'bool',
+        },
         type => {
             schema => ['str*', in=>['function','package']], # XXX other types?
             summary => 'Filter by entity type',
@@ -35,47 +52,61 @@ _
     result_naked => 1,
 };
 sub complete_riap_url {
+    require Complete::Path;
+
     my %args = @_;
 
     my $word = $args{word} // ''; $word = '/' if !length($word);
+    $word = "/$word" unless $word =~ m!\A/!;
+    my $ci          = $args{ci} // $Complete::OPT_CI;
+    my $map_case    = $args{map_case} // $Complete::OPT_MAP_CASE;
+    my $exp_im_path = $args{exp_im_path} // $Complete::OPT_EXP_IM_PATH;
     my $type = $args{type} // '';
 
-    my $scheme;
-    if ($word =~ m!\A/!) {
-        $scheme = '';
+    my $starting_path;
+    my $result_prefix = '';
+    if ($word =~ s!\A/!!) {
+        $starting_path = '/';
+        $result_prefix = '/';
     } elsif ($word =~ s!\Apl:/!/!) {
-        $scheme = 'pl';
+        $starting_path = 'pl:';
+        $result_prefix = 'pl:';
     } else {
         return [];
     }
 
-    my ($pkg, $leaf) = $word =~ m!(.*/)(.*)!;
-    state $pa = do {
-        require Perinci::Access;
-        Perinci::Access->new;
-    };
+    my $res = Complete::Path::complete_path(
+        word => $word,
+        ci => $ci, map_case => $map_case, exp_im_path => $exp_im_path,
+        list_func => sub {
+            my ($path, $intdir, $isint) = @_;
 
-    my $riap_res = $pa->request(list => $pkg, {detail=>1});
-    return [] unless $riap_res->[0] == 200;
+            state $pa = do {
+                require Perinci::Access;
+                Perinci::Access->new;
+            };
 
-    my @res;
-    for my $ent (@{ $riap_res->[2] }) {
-        next unless $ent->{type} eq 'package' ||
-            (!$type || $type eq $ent->{type});
-        next unless index($ent->{uri}, $leaf) == 0;
-        push @res, "$pkg$ent->{uri}";
-    }
+            $path = "/$path" unless $path =~ m!\A/!;
+            my $riap_res = $pa->request(list => $path, {detail=>1});
+            return [] unless $riap_res->[0] == 200;
+            my @res;
+            for my $ent (@{ $riap_res->[2] }) {
+                next unless $ent->{type} eq 'package' ||
+                    (!$type || $type eq $ent->{type});
+                push @res, $ent->{uri};
+            }
+            \@res;
+        },
+        starting_path => $starting_path,
+        result_prefix => $result_prefix,
+        is_dir_func => sub { }, # not needed, we already suffixed "dir" with /
+    );
 
-    # put scheme back on
-    if ($scheme) {
-        for (@res) { $_ = "$scheme:$_" }
-    }
-
-    {words=>\@res, path_sep=>'/'};
+    {words=>$res, path_sep=>'/'};
 }
 
 1;
-#ABSTRACT: Riap-related completion routines
+# ABSTRACT: Riap-related completion routines
 
 __END__
 
@@ -89,7 +120,7 @@ Complete::Riap - Riap-related completion routines
 
 =head1 VERSION
 
-This document describes version 0.01 of Complete::Riap (from Perl distribution Complete-Riap), released on 2014-12-02.
+This document describes version 0.02 of Complete::Riap (from Perl distribution Complete-Riap), released on 2014-12-25.
 
 =head1 SYNOPSIS
 
@@ -111,6 +142,14 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
+=item * B<ci> => I<bool>
+
+Whether to do case-insensitive search.
+
+=item * B<exp_im_path> => I<bool>
+
+=item * B<map_case> => I<bool>
+
 =item * B<type> => I<str>
 
 Filter by entity type.
